require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_advancedauditing',
  dscmeta_resource_friendly_name: 'AdvancedAuditing',
  dscmeta_resource_name: 'AdvancedAuditing',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'cSecurityOptions',
  dscmeta_module_version: '3.1.3',
  docs: 'The DSC AdvancedAuditing resource type.
         Automatically generated from version 3.1.3',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_category: {
      type: "Enum['System;IPsec Driver', 'System;System Integrity', 'System;Security System Extension', 'System;Security State Change', 'System;Other System Events', 'Logon/Logoff;Network Policy Server', 'Logon/Logoff;Other Logon/Logoff Events', 'Logon/Logoff;Special Logon', 'Logon/Logoff;IPsec Extended Mode', 'Logon/Logoff;IPsec Quick Mode', 'Logon/Logoff;IPsec Main Mode', 'Logon/Logoff;Account Lockout', 'Logon/Logoff;Logoff', 'Logon/Logoff;Logon', 'Logon/Logoff;User / Device Claims', 'Object Access;SAM', 'Object Access;Kernel Object', 'Object Access;Registry', 'Object Access;Application Generated', 'Object Access;Handle Manipulation', 'Object Access;File Share', 'Object Access;Filtering Platform Packet Drop', 'Object Access;Filtering Platform Connection', 'Object Access;Other Object Access Events', 'Object Access;Detailed File Share', 'Object Access;Removable Storage', 'Object Access;Central Policy Staging', 'Object Access;Certification Services', 'Object Access;File System', 'Privilege Use;Other Privilege Use Events', 'Privilege Use;Non Sensitive Privilege Use', 'Privilege Use;Sensitive Privilege Use', 'Detailed Tracking;RPC Events', 'Detailed Tracking;DPAPI Activity', 'Detailed Tracking;Process Termination', 'Detailed Tracking;Process Creation', 'Policy Change;Audit Policy Change', 'Policy Change;MPSSVC Rule-Level Policy Change', 'Policy Change;Filtering Platform Policy Change', 'Policy Change;Authorization Policy Change', 'Policy Change;Authentication Policy Change', 'Policy Change;Other Policy Change Events', 'Account Management;Security Group Management', 'Account Management;Distribution Group Management', 'Account Management;Other Account Management Events', 'Account Management;Application Group Management', 'Account Management;Computer Account Management', 'Account Management;User Account Management', 'DS Access;Directory Service Changes', 'DS Access;Directory Service Replication', 'DS Access;Directory Service Access', 'DS Access;Detailed Directory Service Replication', 'Account Logon;Other Account Logon Events', 'Account Logon;Kerberos Service Ticket Operations', 'Account Logon;Credential Validation', 'Account Logon;Kerberos Authentication Service']",
      desc: ' ',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_auditlevel: {
      type: "Enum['No Auditing', 'Success', 'Failure', 'Success and Failure']",
      desc: ' ',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
  },
)
